;##########################
;### Loading Extensions ###
;##########################

extensions [ gis nw csv]
;gis needed for importing geodata, nw for working with networks (pathfinding for example) and csv to generate the output

;################################################
;### Declaring globals, breeds and attributes ###
;################################################

globals [
  gridDataset
  nodeDataset
  linkDataset
]

breed [ people person ]
breed [ trams tram ]
breed [ buses bus ]
breed [ destinations a-destination ]
breed [ tramstops tramstop ]
breed [ recorder record ]
breed [ filler-vehicles filler ]

breed [ nodes node ]
breed [ nlabels nlabel ]

nodes-own [
  isStop
  isType
  trafficPoint
  speedlimit
  car-to-public
]

links-own [
  linkLength
  traffic
  tramline
  maxspeed
]

people-own [
  transport-type
  destination
  transfer-destination
  location
  travel-time
  waiting-time
  actual-time
  delay
  status
  activity-time
  nodeList
  onlyNodes
  onlyNodes-transfer
  pre-walk
  happiness
  willing
  mode
  initial-destination
  start-timestamp
  end-timestamp
]

trams-own [
  line
  direction
  destination
  nodeList
  onlyNodes
  location
  waiting
]

buses-own [
  line
  direction
  destination
  nodeList
  onlyNodes
  location
  waiting
]

filler-vehicles-own [
  destination
  location
  status
  travel-type
  nodeList
  onlyNodes
  delay
]

recorder-own [
  which-agent
  time-taken
  transport-type
  waiting-spent
  travel-spent
  happiness
  willing
]

;############################
;### Setup the simulation ###
;############################

to setup
  ca
  reset-ticks
  ;file-close-all

  setup-labels
  ask patches [ set pcolor white ]

  let grid true
  if nameOfGrid = "" [ set grid false ]
  ;can be specified to include an ascii grid file as a basemap
  ;wasn't used here due to a map flooding the screen with information as well as hindering the readability of moving agents
  read-geodata grid
  display-geodata grid

  ask nodes [
    set speedLimit 0
    if IsStop = 1 [
      set color 52
      set size 8
      ;bus and tram stops are highlighted as larger green points
    ]
  ]

  ask links [
    ifelse tramline = 1 or tramline = 3 [
      set color red
      set thickness 3
      ;highlights tramlines
    ]
    [
      set thickness 1
    ]
    ask one-of links [
      let placeholder maxspeed
      let a end1
      let b end2
      ask a [
        if speedLimit = 0 [
          set speedLimit placeholder
        ]
      ]
      ask b [
        if speedLimit = 0 [
          set speedLimit placeholder
        ]
      ]
      ;nodes get a speedlimit as well
    ]
  ]

  ifelse new-route?
  [ let linkPath []
    let nodePath []
    ask node 219 [
      nw:set-context nodes links
      set linkPath nw:weighted-path-to node 98 linklength
      nw:set-context nodes links
      set nodePath nw:turtles-on-weighted-path-to node 98 linklength
    ]
    foreach linkPath [ x -> ask x [ set color blue set tramline 500 set thickness 3 set traffic 2 ] ]
    foreach nodePath [ x -> ask x [ set isType 500 ] ]

    ask first nodePath [
      set color blue
      set size 6
      set isStop 1
    ]

    ask last nodePath [
      set color blue
      set size 6
      set isStop 1
    ]
    ;the express bus route gets highlighted in blue
  ]
  [ ]

  place-public

  create-recorder 1

  ask recorder [
    setxy -300 -240
    set size 20
    set shape "box"
    set color brown
    set which-agent []
    set time-taken []
    set transport-type []
    set waiting-spent []
    set travel-spent []
    set happiness []
    set willing []
  ]
  ;storing information which can be accessed by inspecting the brown box (bottom left of the map)
end

;########################
;### read the geodata ###
;########################

to read-geodata [ readGrid ]
  ;set variables for datasets
  set nodeDataset gis:load-dataset ("PointsSimple.shp" )
  set linkDataset gis:load-dataset ("LinesSimple.shp" )

  ;_____________________________________________________________________________________________________________________________________________________
  ;##################################################### IMPORTANT: DEFINE PATH ########################################################################
  if readGrid [ set gridDataset gis:load-dataset word "your_path" ( word nameOfGrid ".asc") ]
  ;again, a grid file can be specified here
  ;#####################################################################################################################################################

  ;set extent
  ifelse readGrid [
    gis:set-world-envelope (gis:envelope-union-of (gis:envelope-of gridDataset)
      (gis:envelope-of nodeDataset)
      (gis:envelope-of linkDataset))
  ][
    ;bugfixing envelope. to small -> nodes could be created out of world extent
    let envNodes [] foreach gis:envelope-of nodeDataset [ ?1 -> set envNodes lput ( ?1 + 0.00001 ) envNodes ]
    gis:set-world-envelope (gis:envelope-union-of (envNodes)
      (gis:envelope-of linkDataset))
  ]
end

;###########################
;### display the geodata ###
;###########################

to display-geodata [ displayGrid ]
  ;draw grid
  if displayGrid [ gis:paint gridDataset 0 ]

  ;draw nodes
  let nodescount 0
  foreach gis:feature-list-of nodeDataset [ ?1 -> ;iterate over all features
    let loc gis:location-of (first(first(gis:vertex-lists-of ?1))) ;get location of feature...
    let stopTram gis:property-value ?1 nameofStop ;get length of feature
    let stopType gis:property-value ?1 nameType ;get type of feature
    let feat ?1
      create-nodes 1[
        set xcor item 0 loc ;..and put the node at the same location
        set ycor item 1 loc
        set shape "circle"
        set color red
        set size 0.5
        set isStop stopTram
        set isType stopType
      ]
  ]

  ;generate and draw links for routing
  foreach gis:feature-list-of linkDataset[ ?1 -> ;iterate over all features
    let agent1 gis:property-value ?1 nameOfStartNode
    let agent2 gis:property-value ?1 nameOfEndNode

    let shapeLength gis:property-value ?1 nameoflengthfield ;get length of feature
    let traffictype gis:property-value ?1 nameTrafficType ;get traffic type of feature (either cars only or tram and cars)
    let tramNumber gis:property-value ?1 nameTramLine ;get tram line of feature (either line 1, line 3 or no line)
    let speed gis:property-value ?1 nameSpeed ;get the maximum allowed speed of feature

    ask nodes with [who = agent1] [
      create-link-with node agent2 [
        set linkLength shapeLength
        set traffic traffictype
        set tramline tramNumber
        set maxspeed speed
      ]
    ]
    ;create the links
  ]

end

;####################################
;### Optional: Set up some labels ###
;####################################

to setup-labels

  ifelse load-labels? [
    ask patch -102 100 [
      set plabel "Pfersee"
      set plabel-color black
    ]
    ask patch -95 -190 [
      set plabel "Göggingen"
      set plabel-color pink
    ]
    ask patch 100 12 [
      set plabel "Antonsviertel"
      set plabel-color black
    ]
    ask patch 155 120 [
      set plabel "Hauptbahnhof"
      set plabel-color black
    ]
    ask patch -260 150 [
      set plabel "Stadtbergen"
      set plabel-color orange
    ]
    ;5 labels to improve readability (since this simulation doesn't load in a map)
  ]
  []
end

;##############################
;### creating moving agents ###
;##############################

to generate-people
  create-people number-of-people

  let split-car round (0.7 * count people)
  let split-public round (0.3 * count people)
  ;modal split in the inner city (conducted in 2018) suggests roughly 68.5% using a car while around 31.5% use public transport (this is not including other means
  ;such as bikes, motorcycles and pedestrians)
  ;this simulation will use a 3:7 ratio as an approximation

;### Attributes for car agents ###
  ask n-of split-car people with [transport-type != "public"] [
    set status "wait"
    ;only at the beginning of the simulation
    set transport-type "car"
    set onlyNodes []
    if transport-type = "car" [
      set shape "car"
      set size 13
      set color brown
    ]
    set destination one-of ["Göggingen" "Stadtbergen" ]

    set location one-of nodes with [speedlimit <= 60 and speedlimit != 0 and who != 131 and who != 135]
    move-to location

    set delay random 140 + 20
    ;will be used to not let every agent start at once (only applies to the very beginning of the model!)

    ifelse destination = "Göggingen"
    [ set initial-destination "Göggingen" ]
    [ set initial-destination "Stadtbergen" ]
  ]

;### Attributes for public transport agents ###
  ask n-of split-public people with [transport-type != "car"] [
    set shape "person"
    set status "waiting"
    set size 16
    set transport-type "public"
    ;reclass some people to public transport users
    set destination one-of ["Göggingen" "Stadtbergen" ]

    set location one-of nodes with
    [isStop = 1 and who != 192 and who != 191 and who != 98
    and who != 88 and who != 217 and who != 219 ]

    ifelse destination = "Göggingen"
    [ set color pink ]
    [ set color orange]

    move-to location

    set pre-walk random 240 + 60
    ;people taking tram or bus assume that they needed between 1-5min to get to their nearest stop
  ]

;### Leisure/Work distribution ###
  ask n-of (0.5 * number-of-people) people [
    set mode "work"
  ]
  ask people with [mode != "work"] [
    set mode "leisure"
  ]
  ;splitting 50:50, since the distribution in Augsburg suggests roughly that (important: overall, this might vary from Mo-Fr to Sa-Su)

;### Initial Happiness ###
  ask people [
    set happiness 100
    ;initial value
    set willing "no"
  ]

;### filler vehicles ###
  create-filler-vehicles (0.5 * number-of-people)

  ask filler-vehicles [
    set shape "car"
    set status "wait"
    set size 11
    set color black
    set delay random 500
    let dice random 10
    if dice = 0 or dice = 1 [
      set travel-type "Linie 3"
      set location one-of nodes with [ who = 101 or who = 190 ]
      move-to location
      ifelse location = node 101
      [ set destination node 190 ]
      [ set destination node 101 ]
      ;setting up start and finish
    ]
    if dice = 2 or dice = 3 [
      set travel-type "Linie 1"
      set location one-of nodes with [ who = 254 or who = 190 ]
      move-to location
      ifelse location = node 254
      [ set destination node 190 ]
      [ set destination node 254 ]
      ;setting up start and finish
    ]
;### using a larger range for B17 travelers in order to get a higher traffic flow ####
;    traffic counts imply roughly 32000 vehicles a day on the Gögginger Straße compared to 76000 on the B17 highway
    if dice > 3 and dice < 9 [
      set travel-type "B17"
      set location one-of nodes with [who = 2 or who = 140]
      move-to location
      ifelse location = node 2
      [ set destination node 140 ]
      [ set destination node 2 ]
      ;setting up start and finish
    ]
    if dice = 9 or dice = 10 [
      set travel-type "random"
      set location one-of nodes
      move-to location
      set destination one-of nodes
      ;setting up start and finish
    ]
  ]
end

to place-public

;### Creation of trams ###
  create-trams 4
  ;4 trams circulating from start to finish approximates the frequency seen in the SWA schedules (e.g. roughly 9min interval from Stadtbergen to Königsplatz in Line 3)

  ask trams [
    set shape "train passenger car"
    set size 40
    set color brown
    set onlyNodes []
  ]

  ask one-of trams with [ destination = 0 ] [
    set destination "Königsplatz3"
    set location node 98
    move-to location
    set line 3
  ]
  ask one-of trams with [ destination = 0 ] [
    set destination "Königsplatz1"
    set location node 219
    move-to location
    set line 1
  ]
  ask one-of trams with [ destination = 0 ] [
    set destination "Stadtbergen3"
    set location node 191
    move-to location
    set line 3
  ]
  ask one-of trams with [ destination = 0 ] [
    set destination "Göggingen1"
    set location node 192
    move-to location
    set line 1
  ]
  ;create trams in the start/endpoints

;### turning on the hypothetical busroute (will be called Line 500 here) creates 2 buses ###
  if new-route? [
    create-buses 2
    ask buses [
      set shape "bus"
      set size 35
      set color green
      set onlyNodes []
    ]
    ask one-of buses with [ destination = 0 ] [
      set destination "Göggingen500"
      set location node 98
      move-to location
      set line 500
    ]
    ask one-of buses with [ destination = 0 ] [
      set destination "Stadtbergen500"
      set location node 219
      move-to location
      set line 500
    ]
  ]

end

;##############################################################
;### procedure for shortest path (car and public transport) ###
;##############################################################

to calculate
  ask people with [transport-type = "car"] [ calculate-paths-cars ]
  ask people with [transport-type = "public"] [ calculate-paths-public ]
  ask trams [ calculate-paths-trams ]
  ask buses [ calculate-paths-buses ]
  ask filler-vehicles [ calculate-paths-filler ]
  ;calls the five individual procedures which are necessary
end

;###################################
;### subprocedure for car agents ###
;###################################

to calculate-paths-cars
  let finalNode 0
  if transport-type = "car" [
    if destination = "Stadtbergen" [
      set finalNode 88
    ]
    if destination = "Göggingen" [
      set finalNode 217
    ]
    if is-node? destination [
      set finalNode [who] of destination
    ]
  ]
  ;setting up the last node depending on the destination (cars only)
  let Route []
  let nodePath []
  let totalTime 0
  ask location [
    nw:set-context nodes links
    set Route nw:weighted-path-to node finalNode linklength
    ;provides a list with the links that an agent has to go through to reach its goal
    nw:set-context nodes links
    set totalTime nw:weighted-distance-to node finalNode linklength
    ;calculates the total traveltime
    nw:set-context nodes links
    set nodePath nw:turtles-on-weighted-path-to node finalNode linklength
    ;provides a list with the nodes that an agent has to go through to reach its goal (includes the starting node)
   ]
 set nodeList Route
 set onlyNodes nodePath
 set travel-time totalTime
end

;######################################
;### subprocedure for public agents ###
;######################################

to calculate-paths-public
  let finalNode 0
  if destination = "Stadtbergen" [
    set finalNode 98
  ]
  if destination = "Göggingen" [
    set finalNode 219
  ]
  ;setting up the last node

  let Route []
  let nodePath []
  ask location [
    nw:set-context nodes links with [ traffic = 2 ]
    set Route nw:weighted-path-to node finalNode linklength
    ;provides a list with the links that an agent has to go through to reach its goal
    nw:set-context nodes links with [ traffic = 2 ]
    set nodePath nw:turtles-on-weighted-path-to node finalNode linklength
    ;provides a list with the nodes that an agent has to go through to reach its goal (includes the starting node)
   ]
   set nodeList Route
   set onlyNodes nodePath

   if destination = "Stadtbergen" and member? node 190 onlyNodes [
     set transfer-destination "Königsplatz1"
   ]

   if destination = "Göggingen" and member? node 190 onlyNodes [
     set transfer-destination "Königsplatz3"
   ]

   if new-route? [
     if destination = "Stadtbergen" and transfer-destination = 0 and member? node 246 onlyNodes [
       set transfer-destination "Göggingen500"
     ]

     if destination = "Göggingen" and transfer-destination = 0 and member? node 82 onlyNodes [
       set transfer-destination "Stadtbergen500"
     ]
   ]

   let transfer-node 0

   if destination = "Stadtbergen" and transfer-destination = "Königsplatz1" [
     set transfer-node 192
   ]

   if destination = "Göggingen" and transfer-destination = "Königsplatz3" [
     set transfer-node 191
   ]

   if destination = "Stadtbergen" and transfer-destination = "Göggingen500" [
     set transfer-node 219
   ]

   if destination = "Göggingen" and transfer-destination = "Stadtbergen500" [
     set transfer-node 98
   ]

   if transfer-destination != 0 [
     ask location [
       nw:set-context nodes links with [ traffic = 2 ]
       set nodePath nw:turtles-on-weighted-path-to node transfer-node linklength
       ;provides a list with the nodes that an agent has to go through to reach its goal (includes the starting node)
     ]
     set onlynodes-transfer nodePath
   ]
end

;##############################
;### subprocedure for buses ###
;##############################

to calculate-paths-buses
  let finalNode 0
  if destination = "Stadtbergen500" [
    set finalNode 98
  ]
  if destination = "Göggingen500" [
    set finalNode 219
  ]
  ;setting up the last node (express bus of the new line only)

  let Route []
  let nodePath []
  ask location [
    nw:set-context nodes links with [ traffic = 2 ]
    set Route nw:weighted-path-to node finalNode linklength
    ;provides a list with the links that an agent has to go through to reach its goal
    nw:set-context nodes links with [ traffic = 2 ]
    set nodePath nw:turtles-on-weighted-path-to node finalNode linklength
    ;provides a list with the nodes that an agent has to go through to reach its goal (includes the starting node)
   ]
   set nodeList Route
   set onlyNodes nodePath
end

;##############################
;### subprocedure for trams ###
;##############################

to calculate-paths-trams
  let finalNode 0
  if destination = "Königsplatz3" [
    set finalNode 191
  ]
  if destination = "Königsplatz1" [
    set finalNode 192
  ]
  if destination = "Stadtbergen3" [
    set finalNode 98
  ]
  if destination = "Göggingen1" [
    set finalNode 219
  ]
  ;setting up the last node (trams only)

  let Route []
  let nodePath []
  ask location [
    nw:set-context nodes links with [ traffic = 2 ]
    set Route nw:weighted-path-to node finalNode linklength
    ;provides a list with the links that an agent has to go through to reach its goal
    nw:set-context nodes links with [ traffic = 2 ]
    set nodePath nw:turtles-on-weighted-path-to node finalNode linklength
    ;provides a list with the nodes that an agent has to go through to reach its goal (includes the starting node)
   ]
   set nodeList Route
   set onlyNodes nodePath
end

;#########################################
;### subprocedure for filler-vehicles ###
;#########################################

to calculate-paths-filler
  let finalNode [who] of destination
  let Route []
  let nodePath []
  ask location [
    nw:set-context nodes links
    set Route nw:weighted-path-to node finalNode linklength
    ;provides a list with the links that an agent has to go through to reach its goal
    nw:set-context nodes links
    set nodePath nw:turtles-on-weighted-path-to node finalNode linklength
    ;provides a list with the nodes that an agent has to go through to reach its goal (includes the starting node)
   ]
   set nodeList Route
   set onlyNodes nodePath
end

;####################
;### Go Procedure ###
;####################

to go
;### Condition for end ###
  if ticks = 30000 [
    ask trams [
      ht
    ]
    ask buses [
      ht
    ]
    ask filler-vehicles [
      ht
    ]
    ask people with [status = "travel"] [
      ht
    ]
    ;hides unimportant entities at the very end
    write-output
    stop
  ]
  ;simulation stops after a bit over 8h (1 tick is 1 second)

;### Visualizing capacity ###
  show-capacity
  ;a number besides both buses and trams to show how many people are currently traveling with one

;### Delaying the first trip randomly ###
  ask people with [transport-type = "car" and status = "wait"] [
    ifelse delay <= 0
    [ set status "travel" ]
    [ set delay delay - 1 ]
    if delay = 0 [
      set start-timestamp ticks
    ]
  ]
  ;used to not make every single car agent start at once

  ask filler-vehicles with [ status = "wait" ] [
    ifelse delay <= 0
    [ set status "travel" ]
    [ set delay delay - 1 ]
  ]
  ;used to not make every single filler vehicle start at once

;### Reset the variable "actual-time" ###
  ask people with [transport-type = "car" and (status = "travel" or status = "travel for home")] [
    set actual-time 0
    if length onlyNodes = 1 [
      if distance first onlyNodes >= 2 [
        set end-timestamp ticks
      ]
      ;used to store information for when the trip has ended
    ]
  ]
  ;this resets the time of car users after they did their activity

  ask people with [transport-type = "public" and (status = "travel" or status = "travel for home")] [
    set actual-time 0
  ]
  ;this resets the time of public transport users after they did their activity

;### Set the destination of public transport agents to home ###
  ask people with [transport-type = "public" and transfer-destination = 0 and status = "waiting for home"] [
    set destination "home"
  ]

;### Incrementing the variables for public transport agents (waiting and traveling) ###
  ask people with [transport-type = "public" and (status = "waiting" or status = "waiting for home")] [
    set waiting-time waiting-time + 1
  ]
  ;store and increment time which was spent waiting at busstops/tramstops

  ask people with [transport-type = "public" and (status = "travel" or status = "travel for home")] [
    set travel-time travel-time + 1
  ]

;### BUGFIXING: Prevents false connections with "tie" which can occasionally happen when 1 bus and 1 tram are present at the exact same time ###
  ask people [
    if count my-in-links >= 2 [
      ask my-in-links [ die ]
      ;prevent agents from linking up with one bus and one tram at the same time due to timing (this would cause all three entities to be stuck)
    ]
  ]

;### Very important process: Storing the information about travel/waiting time as well as happiness ###
  ask people with [(status = "activity" or status = "activity at home") and actual-time = 0] [
    let additional-time one-of [ 20 40 60 80 100 120 140 160 180 ]
    ;1-3min for finding parking lot or walking from the bus/tramstop to the destination
    ifelse transport-type = "car"
    [ set actual-time (end-timestamp - start-timestamp) + additional-time ]
    [ set actual-time (end-timestamp - start-timestamp) + additional-time + pre-walk ]

    if additional-time >= 20 and additional-time <= 60 [
      set happiness happiness - (1 * parking-last-loss)
    ]
    if additional-time >= 80 and additional-time <= 120 [
      set happiness happiness - (2 * parking-last-loss)
    ]
    if additional-time >= 140 and additional-time <= 180 [
      set happiness happiness - (3 * parking-last-loss)
    ]
    ;the longer it takes to find a parking lot, the unhappier a person will get

    if transport-type = "public" [
      let total-waiting waiting-time / 60
      set happiness happiness - (total-waiting * waiting-loss)
    ]
    ;deduct happiness based on how long public transport users had to spend waiting

    if happiness < happiness-threshold [
      set willing "yes"
    ]

    if happiness < 0 [
      set happiness 0
    ]
    ;zero happiness is the minimum value

    let id who
    let time actual-time / 60
    ;seconds into minutes
    let happy happiness
    let transp transport-type
    ifelse transport-type = "car"
    [ set transp 1 ]
    [ set transp 2 ]
    ;identifying cars as 1 and public transport as 2 (in order for the output to consist of only numeric values, making the extracting of information easier)

    let change willing
    let waittotal (waiting-time / 60)
    let traveltotal (travel-time / 60)

    ask recorder [
      set which-agent insert-item (length which-agent) which-agent id
      ;insert the id of the agents into the current list
      set time-taken insert-item (length time-taken) time-taken time
      ;insert the time taken of the agents into the current list
      set transport-type insert-item (length transport-type) transport-type transp
      ;insert the happiness at the destination
      set waiting-spent insert-item (length waiting-spent) waiting-spent precision waittotal 3
      ;insert the total time a public transport agent had to wait (will be 0 for car agents)
      set travel-spent insert-item (length travel-spent) travel-spent precision traveltotal 3
      ;insert the total time a public transport agent had to travel (will be 0 for car agents)
      set happiness insert-item (length happiness) happiness precision happy 3
      ;insert the happiness at the destination
      set willing insert-item (length willing) willing change
      ;insert willingness to consider changing mode of transport
    ]
    ifelse mode = "work"
    [ set activity-time random 2000 + 25000 ]
    ;record the time and randomly determine how long the agents spend on their destination
    [ set activity-time random 5000 + 1800 ]
  ]

;### Decrement activity-time each tick ###
  ask people with [ (status = "activity" or status = "activity at home") and activity-time > 0 ] [
    ifelse actual-time > ticks
    []
    [ set activity-time activity-time - 1 ]
  ]

;### Reset shortest-path list, then reverse it to prepare for the trip home ###
  ask people with [ (status = "activity" or status = "activity at home") and activity-time = 0 and transport-type = "car" ] [
    calculate-paths-cars
    ;get the original lists back
    set location last onlyNodes
    ;change the location (lew location is the prior destination)
    set destination first onlyNodes
    calculate-paths-cars
    ;new shortest path for the way back
    set status "travel for home"
    set happiness 100
    set start-timestamp ticks
  ]

  ask people with [ status = "activity" and activity-time = 0 and transport-type = "public" ] [
    set status "waiting for home"
    set happiness 100
    set start-timestamp ticks
    ;reset happiness completely
    set actual-time 0
    set travel-time 0
    set waiting-time 0
    ;reset the seperate times
    calculate-paths-public
    ;get the original path back


    set location last onlyNodes
    move-to location

    let nodePath []
    let transfer-node 0

    if destination != "home" [
      if transfer-destination = "Königsplatz1" [
        set transfer-destination "Königsplatz3"
        set transfer-node 191
        set destination "home"
      ]
    ]

    if destination != "home" [
      if transfer-destination = "Königsplatz3" [
        set transfer-destination "Königsplatz1"
        set transfer-node 192
        set destination "home"
      ]
    ]

    if transfer-destination = "Göggingen500" [
      set destination "home"
      set transfer-node 219
    ]

    if transfer-destination = "Stadtbergen500" [
      set destination "home"
      set transfer-node 98
    ]

    if transfer-destination != 0 [
      ask location [
        nw:set-context nodes links with [ traffic = 2 ]
        set nodePath nw:turtles-on-weighted-path-to node transfer-node linklength
        ;provides a list with the nodes that an agent has to go through to reach its goal (includes the starting node)
      ]
      set onlynodes-transfer nodePath
    ]
  ]

  ask people with [ status = "activity at home" and activity-time = 0 and transport-type = "public" ] [
    ifelse location = node 219
    [ set destination "Göggingen" ]
    [ set destination "Stadtbergen" ]
    set location one-of nodes in-radius 2
    calculate-paths-public
    set status "waiting"
    set start-timestamp ticks
  ]

;### Reset shortest paths for trams, determine their time spent at stops as well as their driving speed ###
  ask trams [
    ifelse empty? onlyNodes = TRUE
    [ calculate-paths-trams
      ;get the original lists back
      set location last onlyNodes
      ;change the location (lew location is the prior destination)
      if location = node 98 [
        set destination "Königsplatz3"
      ]
      if location = node 191 [
        set destination "Stadtbergen3"
      ]
      if location = node 192 [
        set destination "Göggingen1"
      ]
      if location = node 219 [
        set destination "Königsplatz1"
      ]
      calculate-paths-trams
      ;new shortest path for the way back
    ]
    [ face item 0 onlyNodes
      ifelse distance item 0 onlyNodes <= 1
      [ if [isStop] of item 0 onlyNodes = 1 and distance item 0 onlyNodes != 0 [
          let dice one-of [ -3 -2 -1 0 1 2 3 ]
          set waiting wait-time + dice
          ;determine time at the busstop before arriving at one
        ]
        move-to item 0 onlyNodes
        ifelse waiting > 0
        [ set waiting waiting - 1
          ;decrement waiting until it hits 0
        ]
        [ set onlyNodes but-first onlyNodes
          ;get next destination when waittime hits 0
        ]
      ]
      [
        if length onlyNodes = 0 [
          stop
        ]
        ifelse length onlyNodes = 1
        [ let currentSpeed [maxspeed] of last nodeList
          fd (currentSpeed / 30)
        ]
        [ let firstItem [who] of item 0 onlyNodes
          let nextItem [who] of item 1 onlyNodes
          let currentSpeed [maxspeed] of link firstItem nextItem
          let thisDirection heading
          fd (currentSpeed / 30)
        ]
        ;the get-speed procedure wasn't used here due to agents also checking if they have to slow down
        ;having trams also check about slowing down will decrease performance
        ;trams have priority in this model
      ]
    ]
  ]

;### Reset shortest paths for buses as well as their driving speed ###
  ask buses [
    ifelse empty? onlyNodes
    [ calculate-paths-buses
      ;get the original lists back
      set location last onlyNodes
      ;change the location (lew location is the prior destination)
      ifelse destination = "Stadtbergen500"
      [ set destination "Göggingen500" ]
      [ set destination "Stadtbergen500" ]
      calculate-paths-buses
      ;new shortest path for the way back
    ]
    [ face item 0 onlyNodes
      ifelse distance item 0 onlyNodes <= 1
      [ if [isStop] of item 0 onlyNodes = 1 and distance item 0 onlyNodes != 0 [
          set waiting wait-time
          ;determine time at the busstop before arriving at one
        ]
        move-to item 0 onlyNodes
        ifelse waiting > 0
        [ set waiting waiting - 1
          ;decrement waiting until it hits 0
        ]
        [ set onlyNodes but-first onlyNodes
          ;get next destination when waittime hits 0
        ]
      ]
      [
        if length onlyNodes = 0 [
          stop
        ]
        ifelse length onlyNodes = 1
        [ let currentSpeed [maxspeed] of last nodeList
          fd (currentSpeed / 30)
        ]
        [ let firstItem [who] of item 0 onlyNodes
          let nextItem [who] of item 1 onlyNodes
          let currentSpeed [maxspeed] of link firstItem nextItem
          let thisDirection heading
          fd (currentSpeed / 30)
        ]
        ;the get-speed procedure wasn't used here due to agents also checking if they have to slow down
        ;having buses also check about slowing down will decrease performance
        ;buses in this model have priority in traffic, this is for example envisioned in the project Verkehr 4.0 (a seperate lane on the B17 highway
        ;in which buses can travel freely incase congestion occurs)
      ]
    ]
  ]

;### Determine driving speed of car agents ###
  ask people with [transport-type = "car" and status != "wait"] [
    let myDirection heading
    ifelse empty? onlyNodes
    [ if destination = "Göggingen" or destination = "Stadtbergen" [
        set status "activity"
        stop
      ]
      if (destination != "Göggingen" or destination != "Stadtbergen") and distance node 88 > 5 or distance node 217 > 5 [
        set status "activity at home"
        stop
      ]
    ]
    [ face item 0 onlyNodes
      ifelse distance item 0 onlyNodes <= 2 [
        move-to item 0 onlyNodes
        set onlyNodes but-first onlyNodes
      ]
      [
        get-speed
        if count people with [heading = myDirection] in-cone 2 90 > 1 [
          set happiness happiness - congestion-loss
        ]
        ;also includes happiness decreasing if a car has to slow down
      ]
    ]
  ]

;### Determine driving speed of filler-vehicles ###
  ask filler-vehicles with [ status != "wait"] [
    let myDirection heading
    ifelse empty? onlyNodes
    [
      calculate-paths-filler
      move-to location
      set status "wait"
      set delay random 60 + 60
      ;delay the filler vehicles randomly again so that the same traffic pattern doesn't loop over and over again
    ]
    [ face item 0 onlyNodes
      ifelse distance item 0 onlyNodes <= 2 [
        move-to item 0 onlyNodes
        set onlyNodes but-first onlyNodes
      ]
      [
        if length onlyNodes = 0 [
          stop
        ]
        ifelse length onlyNodes = 1
        [ let currentSpeed [maxspeed] of last nodeList
          fd (currentSpeed / 30)
        ]
        [ let firstItem [who] of item 0 onlyNodes
          let nextItem [who] of item 1 onlyNodes
          let currentSpeed [maxspeed] of link firstItem nextItem
          let thisDirection heading
          fd (currentSpeed / 30)
        ]
        ;the get-speed procedure wasn't used here due to agents also checking if they have to slow down
        ;having filler-cars also check about slowing down will decrease performance
      ]
    ]
  ]

;### The following 4 "ask people with" blocks are needed to:
;    - link up waiting people with the correct public transport vehicle
;      (someone going to Königsplatz needs to take the bus that goes there, not the one which leaves from the Königsplatz
;    - remove the link once they reach either their transfer destination or final destination
;    - move from their transfer-destination to the other line (only happens at the Königsplatz)

;### (1) Includes transfering for the trip towards the destination
  ask people with [transport-type = "public" and transfer-destination != 0 and destination != "home"
  and status != "activity" and status != "activity at home" and status != "waiting for home" and status != "might change to car"] [
    let id who
    let lastNode last onlynodes-transfer

    ifelse any? trams in-radius 4 with [empty? onlyNodes]
    [ ask my-in-links [ die ]
      if status = "travel" [
        move-to lastNode
        ;agents dont quite exit the tram/bus perfectly at their destination node
        set status "transfering"
      ]
    ]
    [ if any? trams in-radius 4 with [last onlyNodes = lastNode and empty? onlyNodes = FALSE] [
        create-link-with one-of trams in-radius 4 with [last onlyNodes = lastNode] [ tie ]
        set status "travel"
      ]
    ]

    ifelse any? buses in-radius 4 with [empty? onlyNodes]
    [ ask my-in-links [ die ]
      if status = "travel" [
        move-to lastNode
        ;agents dont quite exit the tram/bus perfectly at their destination node
        set status "transfering"
      ]
    ]
    [ if any? buses in-radius 4 with [last onlyNodes = lastNode and empty? onlyNodes = FALSE] [
        create-link-with one-of buses in-radius 4 with [last onlyNodes = lastNode] [ tie ]
        set status "travel"
      ]
    ]

    if distance lastNode = 0 and status = "transfering" [
      set onlynodes-transfer 0
      set transfer-destination 0
      if status = "transfering" and distance node 191 = 0 [
        move-to node 192
        set happiness happiness - transfer-loss
        ;transfering decreases happiness
        set status "waiting"
      ]
      if status = "transfering" and distance node 192 = 0 [
        move-to node 191
        set happiness happiness - transfer-loss
        ;transfering decreases happiness
        set status "waiting"
      ]
      if status = "transfering" and distance node 219 = 0 or distance node 98 = 0 [
        set happiness happiness - transfer-loss
        ;transfering decreases happiness
        set status "waiting"
      ]
    ]
  ]

;### (2) Includes the trip towards the final destination after agents have transfered once ###
  ask people with [transport-type = "public" and transfer-destination = 0 and destination != "home"
  and status != "activity" and status != "activity at home" and status != "waiting for home" and status != "might change to car"] [
    let id who
    let lastNode last onlyNodes

    ifelse any? trams in-radius 4 with [empty? onlyNodes] and distance lastNode < 10
    [ ask my-in-links [ die ]
      if status = "travel" [
        move-to lastNode
        ;agents dont quite exit the tram/bus perfectly at their destination node
        set end-timestamp ticks
        set status "activity"
      ]
    ]
    [ ifelse any? trams in-radius 4 with [ empty? onlyNodes ]
      [ ]
      [ if any? trams in-radius 4 with [last onlyNodes = lastNode and empty? onlyNodes = FALSE] [
          create-link-with one-of trams in-radius 4 with [last onlyNodes = lastNode] [ tie ]
          set status "travel"
        ]
      ]
    ]

    if distance node 98 < 2 or distance node 219 < 2 [
      ifelse any? buses in-radius 4 with [empty? onlyNodes] and distance lastNode < 10
      [ ask my-in-links [ die ]
        if status = "travel" [
          move-to lastNode
          ;agents dont quite exit the tram/bus perfectly at their destination node
          set end-timestamp ticks
          set status "activity"
        ]
      ]
      [ ifelse any? buses in-radius 4 with [ empty? onlyNodes ]
        [ ]
        [ if any? buses in-radius 4 with [last onlyNodes = lastNode and empty? onlyNodes = FALSE] [
            create-link-with one-of buses in-radius 4 with [last onlyNodes = lastNode] [ tie ]
            set status "travel"
          ]
        ]
      ]
    ]
  ]

;### (3) Includes transfering for the trip home ###
  ask people with [transport-type = "public" and transfer-destination != 0 and destination = "home"
  and status != "activity" and status != "activity at home" and status != "might change to car"] [
    let id who
    let lastNode last onlynodes-transfer

    ifelse any? trams in-radius 4 with [empty? onlyNodes]
    [ ask my-in-links [ die ]
      if status = "travel for home" [
        move-to lastNode
        ;agents dont quite exit the tram/bus perfectly at their destination node
        set status "transfering for home"
      ]
    ]
    [ if any? trams in-radius 4 with [last onlyNodes = lastNode and empty? onlyNodes = FALSE] [
        create-link-with one-of trams in-radius 4 with [last onlyNodes = lastNode] [ tie ]
        set status "travel for home"
      ]
    ]

    ifelse any? buses in-radius 4 with [empty? onlyNodes]
    [ ask my-in-links [ die ]
      if status = "travel for home" [
        move-to lastNode
        ;agents dont quite exit the tram/bus perfectly at their destination node
        set status "transfering for home"
      ]
    ]
    [ if any? buses in-radius 4 with [last onlyNodes = lastNode and empty? onlyNodes = FALSE] [
        create-link-with one-of buses in-radius 4 with [last onlyNodes = lastNode] [ tie ]
        set status "travel for home"
      ]
    ]

    if distance lastNode = 0 and status = "transfering for home" [
      set onlynodes-transfer 0
      set transfer-destination 0
      if status = "transfering for home" and distance node 191 = 0 [
        move-to node 192
        set happiness happiness - transfer-loss
        ;transfering decreases happiness
        set status "waiting for home"
      ]
      if status = "transfering for home" and distance node 192 = 0 [
        move-to node 191
        set happiness happiness - transfer-loss
        ;transfering decreases happiness
        set status "waiting for home"
      ]
      if status = "transfering for home" and distance node 219 = 0 or distance node 98 = 0 [
        set happiness happiness - transfer-loss
        ;transfering decreases happiness
        set status "waiting for home"
      ]
    ]
  ]

;### (4) Includes the trip home after transfering (or if the agent didn't need to transfer to begin with) ###
  ask people with [transport-type = "public" and transfer-destination = 0 and destination = "home"
  and status != "activity" and status != "activity at home" and status != "might change to car"] [
    let id who
    let homeNode first onlynodes

    ifelse any? trams in-radius 4 with [empty? onlyNodes]
    [ ]
    [ if any? trams in-radius 4 with [member? homeNode onlyNodes and empty? onlyNodes = FALSE] [
        create-link-with one-of trams in-radius 4 with [member? homeNode onlyNodes] [ tie ]
        set status "travel for home"
      ]
    ]

    ifelse distance homeNode < 2
    [ ask my-in-links [ die ]
      move-to homeNode
      set destination 0
      set end-timestamp ticks
      set status "activity at home"]
    [ ]
  ]

;### Car agents considering to swap to public transport ###

  ask people with [transport-type = "car" and status = "activity at home" and willing = "yes"] [
    let closest min-one-of nodes with [isStop = 1] [distance myself]
    if distance closest <= 25 [
      set color green
      set size 15
      set shape "x"
      create-link-with closest [set thickness 0.5 set color green]
      set status "might change to public"
      ;distance = 25 translates to roughly 200m in the real world
    ]
    if distance closest > 25 and distance closest <= 37.5 [
      set color yellow
      set size 15
      set shape "x"
      create-link-with closest [set thickness 0.5 set color yellow]
      set status "might change to public"
      ;distance = 37.5 translates to roughly 300m in the real world
      ;having to walk more than 300m to reach a bus/tramstop is indicated to make people not choose public transport
    ]
    if distance closest >= 37.5 [
      set color black
      set size 15
      set shape "x"
      set status "might change to public"
      ;having to walk more than 300m to reach a bus/tramstop is indicated to make people not choose public transport
      ;however, this is shown here to point out the car agents who would want to change to public transport but aren't within a reasonable range
    ]
  ]

;### Public transport agents considering to swap to car ###

  ask people with [transport-type = "public" and status = "activity at home" and willing = "yes"] [
    let closest min-one-of nodes with [isStop = 1] [distance myself]
    set color red
    set size 15
    set shape "star"
    create-link-with closest [set thickness 1.5 set color 52]
    set status "might change to car"
    stop
  ]

;### Visualization of car agents choosing which tram/busstop ###

  ask nodes with [isStop = 1] [
    set label count my-in-links with [thickness = 1.5]
    set label-color white
    ;label number indicates the amount of people from that station who would consider changing from public transport to car
    set car-to-public count my-in-links with [thickness = 0.5]
    ;meanwhile this attribute indicates the amount of people who would consider changing from car to public transport and are within a reasonable range of a bus/tramstop
  ]
  tick
end

;#####################################
;### Procedure for car agents only ###
;#####################################

to get-speed
  if length onlyNodes = 0 [
    stop
  ]
  let myDirection heading
  ifelse length onlyNodes = 1
  [ let currentSpeed [maxspeed] of last nodeList
    ifelse count turtles with [heading = myDirection and (breed = "filler-vehicles" or (breed = "people" and transport-type = "car"))] in-cone 2 45 > 1
    [ fd (0.7 * (currentSpeed / 30)) ]
    ;30% penalty due to cars or filler-vehicles being in the cone (agent slows down)
    [ fd (currentSpeed / 30) ]
  ]
  [ let firstItem [who] of item 0 onlyNodes
    let nextItem [who] of item 1 onlyNodes
    let currentSpeed [maxspeed] of link firstItem nextItem
    let thisDirection heading
    ifelse count turtles with [heading = myDirection and (breed = "filler-vehicles" or (breed = "people" and transport-type = "car"))] in-cone 2 45 > 1
    [ fd (0.8 * (currentSpeed / 30)) ]
    [ fd (currentSpeed / 30) ]
  ]
end

;##########################################
;### Procedure for showing the capacity ###
;##########################################

to show-capacity
  ask buses [
    set label count my-in-links
    set label-color black
  ]
  ask trams [
    set label count my-in-links
    set label-color black
  ]
end

;############################
;### Procedure for output ###
;############################

to write-output
  if new-route? [
    csv:to-file "withbus.csv" [ (list which-agent "\n" time-taken "\n" transport-type "\n" waiting-spent "\n" travel-spent "\n" happiness) ] of recorder
  ]
  if new-route? = FALSE [
    csv:to-file "withoutbus.csv" [ (list which-agent "\n" time-taken "\n" transport-type "\n" waiting-spent "\n" travel-spent "\n" happiness) ] of recorder
  ]
  ;Note: The csv will be replaced/rewritten with each run
end
@#$#@#$#@
GRAPHICS-WINDOW
3
10
642
520
-1
-1
1.0
1
10
1
1
1
0
0
0
1
-315
315
-250
250
0
0
1
ticks
30.0

BUTTON
662
159
725
192
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SWITCH
663
74
777
107
new-route?
new-route?
0
1
-1000

SWITCH
787
74
907
107
load-labels?
load-labels?
0
1
-1000

SLIDER
662
242
834
275
number-of-people
number-of-people
50
300
200.0
10
1
NIL
HORIZONTAL

BUTTON
843
242
944
275
place people
generate-people
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

INPUTBOX
5
551
125
611
nameOfGrid
NIL
1
0
String

INPUTBOX
5
615
125
675
nameOfNodeShape
PointsSimple
1
0
String

INPUTBOX
133
616
253
676
nameOfLinkShape
LinesSimple
1
0
String

INPUTBOX
133
551
254
611
nameOfLengthField
RealDist
1
0
String

INPUTBOX
265
552
375
612
nameOfStartNode
Node1
1
0
String

INPUTBOX
265
617
372
677
nameOfEndNode
Node2
1
0
String

INPUTBOX
385
553
482
613
nameTrafficType
Traffic
1
0
String

INPUTBOX
385
618
476
678
nameTramLine
TramLine
1
0
String

INPUTBOX
490
553
565
613
nameofStop
IsStop
1
0
String

BUTTON
1111
473
1174
506
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

INPUTBOX
489
619
564
679
nameType
IsType
1
0
String

BUTTON
662
308
754
341
calculate paths
calculate
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
661
406
807
439
wait-time
wait-time
15
60
30.0
1
1
s
HORIZONTAL

INPUTBOX
572
553
651
613
nameSpeed
Speed
1
0
String

TEXTBOX
664
12
814
44
(1) Choose scenario with the switches down below
13
0.0
1

TEXTBOX
682
58
832
76
adds busroute
12
0.0
1

TEXTBOX
681
108
831
126
ignores busroute
12
0.0
1

TEXTBOX
661
137
863
155
(2) Create the area of interest
13
0.0
1

TEXTBOX
661
208
907
240
(3) Select your sample size with the slider, then place them
13
0.0
1

TEXTBOX
661
285
969
303
(4) Calculate the shortest path using Dijkstra
13
0.0
1

TEXTBOX
1109
448
1282
466
(7) Start the simulation
15
15.0
0

TEXTBOX
659
368
933
400
(5) Select waiting time of public transportation (how much time is spent at a bus/tramstop)
13
0.0
1

TEXTBOX
657
449
1137
468
(6) Select happiness threshold and impact of sources (all units below match)
13
0.0
1

SLIDER
662
470
807
503
happiness-threshold
happiness-threshold
0
100
50.0
1
1
NIL
HORIZONTAL

SLIDER
837
470
949
503
transfer-loss
transfer-loss
5
15
10.0
1
1
NIL
HORIZONTAL

SLIDER
948
470
1079
503
congestion-loss
congestion-loss
0.2
2
1.2
0.05
1
NIL
HORIZONTAL

SLIDER
837
502
952
535
waiting-loss
waiting-loss
0.20
0.70
0.5
0.05
1
NIL
HORIZONTAL

SLIDER
951
502
1079
535
parking-last-loss
parking-last-loss
2
5
4.0
0.1
1
NIL
HORIZONTAL

TEXTBOX
8
528
276
546
Fields of the attributes stored in the Shapefile:
13
0.0
1

TEXTBOX
784
57
934
75
some labels for readability
12
0.0
1

MONITOR
974
39
1138
84
from car to public transport
count people with [transport-type = \"car\" and willing = \"yes\"]
17
1
11

MONITOR
1137
39
1301
84
from public transport to car
count people with [transport-type = \"public\" and willing = \"yes\"]
17
1
11

TEXTBOX
975
10
1125
28
Monitors
14
95.0
1

TEXTBOX
977
198
1127
216
Plot
14
126.0
1

PLOT
975
222
1328
413
Car agents willing to change
Time in ticks [1 tick = 1s]
Amount of agents
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"<200m from stop" 1.0 0 -11085214 true "" "plot count people with [status = \"might change to public\" and shape = \"x\" and color = 55]"
"<300m from stop" 1.0 0 -1184463 true "" "plot count people with [status = \"might change to public\" and shape = \"x\" and color = 45]"
">300m from stop" 1.0 0 -16777216 true "" "plot count people with [status = \"might change to public\" and shape = \"x\" and color = 0]"

MONITOR
974
89
1137
134
Average happiness
mean [happiness] of one-of recorder
3
1
11

TEXTBOX
1057
158
1207
176
NIL
12
0.0
1

@#$#@#$#@
## WHAT IS IT?

This is a model which simulates car and bus/tram traffic in Augsburg, specifically between the districts Stadtbergen, Göggingen and the Innenstadt. People either use their cars or public transport to travel to one of their random destinations (Stadtbergen or Göggingen), performing some activity and then returning to their home. Both the actual travel time as well as their happiness upon arriving are stored and have an impact on individuals on whether they would consider changing their mode of transport or not. 

## HOW IT WORKS

**World Generation**

Using the gis extension, this model loads in a simplified road network with the roads being links and the nodes being a stationary breed. Both inherit some data such as their calculated length from OSM data in ArcGis Pro, their maximum allowed speed limit and data about the node being a bus/tramstop or not. Both the tram and hypothetical busroute are highlighted with thicker and colored links.

**Agent Placement**

People can either move to their destination by car or using public transport, the ratio is determined from the modal split in the city of Augsburg. Car users can be placed anywhere except certain nodes such as the B17 highway, while public transport users will only be placed on bus/tramstops.

The trams and buses will always start from their initial location (Königsplatz, Stadtbergen or Göggingen).

Additionally, another breed called "filler-vehicles" will be placed on either the north/south end of the B17 highway, the Königsplatz, Stadtbergen and Göggingen as well as some randomly placed cars. Their purpose is to fill up the world with additional traffic especially on the B17 highway and the two main streets towards the Königsplatz replicating what traffic count data in Augsburg suggests.

**Pathfinding**

Using the nw extension, a Djikstra-algorithm is performed to find the shortest path from the starting point of people and trams/buses to their destinations. While the people generate at random nodes each simulation and have a random destination, the buses/trams stay the same for every simulation (move from initial stop to last stop and then back).

The path is stored as a list containing the nodes an agent has to pass to arrive at its destination. If an agent uses public transport and has to transfer to another bus/tramline, then this agent will have two lists (one until the transfer and another one for the whole route).

**People behaviour**

Once the simulation starts, both car users and public transport users will move towards their destinations along the links. 

*Cars*

Car users will drive along the links according to the maximum allowed speed limit. If a lot of cars are infront of the car agent, then it will slow down a bit and cause a bit of their happiness to decrease. The list containing the nodes will get shorter and shorter depending on how much is left until the final destination. Once one car agent reaches their destination, they will stop moving and get some additional time added to include finding a parking lot (this is randomly picked). After that, they will stay at their destination performing some sort of activity such as working which is a counter that will decrease every tick. Upon reaching 0, the agent will first calculate its original list back, swap the current location with its destination and then reverse the node list (this is essentially the way back).

*Public transport Users*

People using public transport will be generated on a random node (except destination nodes and transfer nodes) which is also a tram or busstop. A random amount of time will be generated for every public transport agent to include an approximation of the time they need to walk from their home to the bus or tramstop (a few minutes). They will wait until a bus or tram enters their short radius and check if either their destination or transfer-destination matches the last node of the tram/bus (reminder: every moving agent has its list of nodes that it follows to reach its destination). If they don't match, the public transport agent will wait on the stop. However, if they do match, they will link together using "tie" which is provided by Netlogo. Once public agents reach their destination or transfer-destination, they will untie with the bus or tram they were traveling on. If they reached their final destination, they will first add some time to include their walk from the stop to their actual destination (a few minutes usually) and then begin to perform their activity. However, if they need to transfer to another tramline or busline (e.g. leaving a tram in Line 1 at Königsplatz to enter another tram in Line 3 to travel towards Stadtbergen) they will first move towards the next stop, wait there and then link with the tram or bus that meets criteria of having the same last node in their list as the destination node of the public transport agent. The procedure for getting home after having performed an activity is similar to the one the car agents use (recalculating the old list back, then reverse it). However, one difference is the fact that the destination is a random node (their original starting point) now and not the ending node of a bus or tramline. Therefore, the public transport agent checks if their destination is somewhere in the list of the bus or tram that it needs to take. The agent will tie once again if this is the case and untie once they are near the destination (home) again.

**Tram/Bus/Filler-vehicles behaviour**

Both trams and buses belong to one line and have their own list of nodes that they need to travel through to reach their final destination. In this simplified model, the destinations are located in Göggingen, Stadtbergen and the Königsplatz.

*Trams*

There are 4 trams in total with 2 on each line. Those will travel from their starting point to their endstation while stopping for a short amount of time, when they are on a tramstop along the way (marked as slightly larger green points).

*Buses*

If the hypothetical line (called line 500 in this example) is generated as well, then two buses will travel similarly to the trams (back and forth).

*Filler-vehicles*

Similarly to car agents, this group starts at a location and is assigned a destination. They serve as a means to fill up the overall traffic in this model with a focus on the two main streets towards the Königsplatz as well as the B17 highway, replicating what traffic count data suggests for atleast the B17 and the Gögginger Straße. Once those vehicles reach their goal, they will be moved back to their inital location and drive towards their destination once again.

**Output**

Every time a car or public transport agent reaches its destination, it will store some information in a stationary reporter. This includes:
- the ID of the current agent
- how many minutes it took in total (includes time spent waiting, traveling and going to/from the bus/tram stop or finding a parking lot)
- what mode of transport the agent is using
- time spent waiting (only important for public transport agents)
- time spent traveling (units in minutes for public transport agents and hours for car agents)
- the happiness upon arriving at the desired destination

A procedure writing the values of the attributes of the reporter starts at 30000 ticks (right at the end) in a .csv file.

**Note:** If one wishes to run the model multiple times, the .csv files need to be moved to a different directory since this model would overwrite an already existing .csv.


## HOW TO USE IT

The sequence of the buttons and sliders is shown in the Interface tab with numbers inside brackets "()":

**(1)** Choose the scenario with the switches down below

**- new-route?** 
	**on:** includes the hypothetical express bus route between Stadtbergen and Göggingen (Line 500)
	**off:** doesn't include the express bus route (status quo)

**- load-labels?**
	**on:** load a few labels (for better readability/orientation)
	**off:** don't load in any labels 

**(2)** Create the area of interest

**setup:** generates the Netlogo world by loading in the needed geodata as well as including the new express bus route if it has been chosen in (1)

**(3)** Select sample size of car and public transport agents, then place them randomly

**number-of-people** specifies the amount of car and public transport agents (in total), the ratio equates to the modal split of the city of Augsburg

**place people** places the people on random nodes (with some exceptions such as the B17 highway)

**(4)** Calculate the shortest path using Dijkstra

**calculate paths** calculates the shortest distance using the Dijkstra-algorithm included in the nw-extension and stores the list of nodes that need to be passed until the destination

**(5)** Select time a bus or tram stops at a stop

**wait-time** specifies the amount of ticks (seconds) a mode of public transport will spend at a stop

**(6)** Select happiness threshold and the impact of sources which decrease happiness over the course of the simulation

**happiness-threshold** specifies a threshold (a number between 1 and 100) in which an agent will consider switching its mode of transport if its happiness is lower than the threshold after a journey

**transfer-loss** specifies the reduction in happiness for when public transport agents have to transfer from one tram or bus to another one to reach their destination (same units as happiness-threshold)

**congestion-loss** specifies the reduction in happiness for when car agents have to slow down due to other cars or buses/trams being ahead of them (same units as happiness-threshold)

**waiting-loss** specifies the reduction in happiness for when public transport agents have to wait for a bus/tram to arrive (same units as happiness-threshold)

**parking-last-loss** specifies the reduction in happiness for when both car and public transport agents have to either find a parking lot (car only) or travel from the final bus/tramstop to their destination (public transport only) (same units as happiness-threshold)

**(7)** Start the simulation

**go:** Simulation will start and run once this button is pressed (currently, a piece of code will make it stop once a bit over 8 hours have passed; equivalent to 30000 ticks)

## THINGS TO NOTICE

This simulation focuses more on the final output e.g. the overall travel time, the overall happiness and the percentage of people who are considering to switch their mode of transport. However, if one chooses to observe the simulation, then there are some things to notice:

- the overall traffic flow of the study area as well as which roads are used more often and which aren't

- the monitors as well as the plot to get an idea of the state (average happiness of the people after their trips, how many are willing to change from car to public transport and vice versa)

- (not used in the experimental setup) the number next to buses/trams since this indicates the amount of people that are currently travelling (this could be helpful for extensions to evaluate the capacity of the new express bus line for example)

## THINGS TO TRY

The more interesting slides are the ones in **(5)** and **(6)** since they affect both the travel time and the happiness. With accurate empirical data, those could be adjusted to better represent the happiness of the agents.

## EXTENDING THE MODEL

This simulation is very simple baseline of representing the traffic flow and happiness of the agents in this particular area of interest. This model can be refined and extended in the following ways:

- use a more **accurate traffic network** which includes more roads, traffic lights on important crossovers, the introduction of one-way streets (e.g. the tramline 3 splits up in Pfersee due to two one-way streets) and the introduction of remaining tramlines and already existing buslines (however, this will increase the computation time for both the model and the pathfinding algorithm)

- introduce **more participants** concerning the **mode of transport** such as motorcycles, pedestrians, bikes, e-scooters and buses

- additionally, add agents without a destination into this model; since traffic isn't just a system that is completely limited to the study area, it would be more realistic to have cars travel in and out of the model (e.g. people that drive through the B17)

- apply more accurate **empirical data** concerning the actual modal split in this particular region, the variety in cars (different types of fuel, possibly e-cars), socio-economic (estimate on salaries, how many people actually do own a car, which age groups are present since for example a dominance of old age people might lead to far less cars being used and more public transport; age is also important to understand what kind of school/university and workplace traffic flow this region can expect) and psychological (happiness of the current status quo concerning how fast and comfortable people can reach their destination in this area of interest, happiness concerning the pricing, happiness concerning the comfort on both bus/tramstops and inside buses/trams)

- more room for **interaction between the agents** such as cars stopping at traffic lights leading to a queue of cars, stopping to let pedestrians pass or cars waiting for buses or trams in one-lane streets

## NETLOGO FEATURES

**Netlogo extensions:**
- nw
- gis
- csv

Due to how links work as a datatype in Netlogo, a solution was needed to make the agents move in realistic steps in between the links (move-to would just make them jump from one end to another), which are essentially the entire world they can travel in. Using the **nw** extension proved to be very helpful not only for the calculation of the shortest path but also as a possibility to provide the agents with a list of sequential nodes.

## RELATED MODELS

Related interest:

[1] Simulation of a small town with interactions between pedestrians, traffic lights and vehicles by Jiri Lukas

Netlogo Community Hub Link:
- http://ccl.northwestern.edu/netlogo/models/community/Town%20-%20Traffic%20&%20Crowd%20simulation

[2] Project Autonomous Bus in the
“Innovationspark Augsburg” by Katharina Fogelberg, Sandra Kempter, Johannes Krämer, Tanja Pfeffer and Julia Subal in 2018

## CREDITS AND REFERENCES

Copyright 2021 Eduard Rech.
This model was created at the Institute of Geography, University of Augsburg.

Other references:

[1] Netlogo NW Extension for Network Analysis

Github Link with information about computation time etc.:
- https://github.com/NetLogo/NW-Extension

Netlogo Manual Link:
- https://ccl.northwestern.edu/netlogo/docs/nw.html

[2] Dijkstra by Matthias Benedek in 2014 (Model created at the Institute of Geography, Augsburg)
- generating the links and nodes from geodata used as a basis from this model
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

bus
false
0
Polygon -7500403 true true 15 206 15 150 15 120 30 105 270 105 285 120 285 135 285 206 270 210 30 210
Rectangle -16777216 true false 36 126 231 159
Line -7500403 false 60 135 60 165
Line -7500403 false 60 120 60 165
Line -7500403 false 90 120 90 165
Line -7500403 false 120 120 120 165
Line -7500403 false 150 120 150 165
Line -7500403 false 180 120 180 165
Line -7500403 false 210 120 210 165
Line -7500403 false 240 135 240 165
Rectangle -16777216 true false 15 174 285 182
Circle -16777216 true false 48 187 42
Rectangle -16777216 true false 240 127 276 205
Circle -16777216 true false 195 187 42
Line -7500403 false 257 120 257 207

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

train passenger car
false
0
Polygon -7500403 true true 15 206 15 150 15 135 30 120 270 120 285 135 285 150 285 206 270 210 30 210
Circle -16777216 true false 240 195 30
Circle -16777216 true false 210 195 30
Circle -16777216 true false 60 195 30
Circle -16777216 true false 30 195 30
Rectangle -16777216 true false 30 140 268 165
Line -7500403 true 60 135 60 165
Line -7500403 true 60 135 60 165
Line -7500403 true 90 135 90 165
Line -7500403 true 120 135 120 165
Line -7500403 true 150 135 150 165
Line -7500403 true 180 135 180 165
Line -7500403 true 210 135 210 165
Line -7500403 true 240 135 240 165
Rectangle -16777216 true false 5 195 19 207
Rectangle -16777216 true false 281 195 295 207
Rectangle -13345367 true false 15 165 285 173
Rectangle -2674135 true false 15 180 285 188

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.1.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="test" repetitions="1" runMetricsEveryStep="true">
    <setup>setup
generate-people
calculate</setup>
    <go>test</go>
    <metric>count people with [status = "travel"]</metric>
    <enumeratedValueSet variable="nameOfEndNode">
      <value value="&quot;Node2&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nameOfGrid">
      <value value="&quot;&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nameOfLengthField">
      <value value="&quot;RealDist&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="weekday?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="number">
      <value value="269"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="new-route?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="wait-time">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="number-of-people">
      <value value="200"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="price-litre">
      <value value="1.42"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nameOfNodeShape">
      <value value="&quot;PointsSimple&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="draw-patches?">
      <value value="&quot;no&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nameTramLine">
      <value value="&quot;TramLine&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="visible">
      <value value="&quot;all&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nameOfStartNode">
      <value value="&quot;Node1&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nameType">
      <value value="&quot;IsType&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nameSpeed">
      <value value="&quot;Speed&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nameOfLinkShape">
      <value value="&quot;LinesSimple&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nameofStop">
      <value value="&quot;IsStop&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="load-labels?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="nameTrafficType">
      <value value="&quot;Traffic&quot;"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
